#!/usr/bin/python
# -*- coding: utf-8 -*-

import getopt
import sys
import logging

import urllib.parse as urllib_parse
# from requests_html import HTMLSession

# Importazioni compatibili con Python 2 e 3
if sys.version_info[0] < 3:
    import cookielib
    import urllib2
    from urllib2 import Request, urlopen, HTTPError, URLError
    from urllib import urlencode
else:
    import urllib.request as urllib2
    from urllib.error import HTTPError, URLError
    from urllib.parse import urlencode
    import http.cookiejar as cookielib

# def getHiddenRandHTMLResponse(response):
    # target = "<input type=\"hidden\" name=\"_token\" value=\""
    # targetresponse = "<div id=\"sign-up-wrap\""
    # parsedres = response[response.find(targetresponse): len(response)]
    # return parsedres[parsedres.find(target) + len(target): parsedres.find(target) + len(target) + 40]

# Importazioni compatibili con Python 2 e 3
if sys.version_info[0] < 3:
    # Python 2
    from StringIO import StringIO
else:
    # Python 3
    from io import StringIO


def getHiddenRandHTMLResponse(response):
    # Assicurati che `response` sia una stringa
    if isinstance(response, bytes):
        response = response.decode('utf-8')

    target = '<input type="hidden" name="_token" value="'
    targetresponse = '<div id="sign-up-wrap"'

    # Trova l'inizio della risposta target
    start_index = response.find(targetresponse)
    if start_index == -1:
        return None  # Target non trovato

    # Prendi la sottostringa a partire dal targetresponse
    parsedres = response[start_index:]

    # Trova l'inizio del target
    start_index = parsedres.find(target)
    if start_index == -1:
        return None  # Target non trovato

    # Trova la fine del valore del token
    start_index += len(target)
    end_index = parsedres.find('"', start_index)

    if end_index == -1:
        return None  # Delimitatore di fine non trovato

    # Estrai il valore del token
    return parsedres[start_index:end_index]


def checkLogin(response):
    target = "<title>No-IP Members Portal: Your No-IP</title>"
    if response.find(target) == -1:
        return False
    return True


def usage():
    print("usage: ./noipAutoLogin [options]")
    print("")
    print("options:")
    print("-h, --help 	 show this help message and exit")
    print("-u, --username   set your NoIP login_username")
    print("-p, --password   set your NoIP login_password")
    print("")
    print("example:")
    print("./noipAutoLogin -u username -p password")


class HTMLSession:
    cj = None
    opener = None
    txHeaders = None

    def __init__(self, txHeaders):
        # La CookieJar memorizzerà i cookie necessari durante il processo di login.
        self.cj = cookielib.CookieJar()
        self.txHeaders = txHeaders
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cj))
        urllib2.install_opener(self.opener)

    # Esempio di uso
    # txHeaders deve essere definito prima di passarlo al costruttore.
    # obj = YourClass(txHeaders)


    def setHeaders(self, txheaders):
        self.txHeaders = txheaders

    def getHeaders(self):
        return self.txHeaders

    # def openURI(self, uri, txdata):
        # try:
            # req = urllib2.Request(uri, txdata, self.txHeaders)
            # # create a request object
            # handle = urllib2.urlopen(req)
            # # and open it to return a handle on the url
        # except IOError as e:
            # print('we failed to open "%s".' % uri)
            # if hasattr(e, 'code'):
                # print('We failed with error code - %s.' % e.code)
                # logging.error('We failed with error code - %s.' % e.code)
            # elif hasattr(e, 'reason'):
                # print("The error object has the following 'reason' attribute :")
                # print(e.reason)
                # print("This usually means the server doesn't exist,'")
                # print("is down, or we don't have an internet connection.")
                # return None
        # else:
            # return handle.read()

    def openURI(self, uri, txdata):
        try:
            # In Python 3, txdata should be in bytes
            if isinstance(txdata, str):
                txdata = txdata.encode('utf-8')

            req = urllib2.Request(uri, txdata, self.txHeaders)
            # Create a request object
            handle = urllib2.urlopen(req)
            # Open it to return a handle on the URL
        except (HTTPError, URLError) as e:
            print('We failed to open "%s".' % uri)
            if hasattr(e, 'code'):
                print('We failed with error code - %s.' % e.code)
                logging.error('We failed with error code - %s.' % e.code)
            elif hasattr(e, 'reason'):
                print("The error object has the following 'reason' attribute:")
                print(e.reason)
                print("This usually means the server doesn't exist,")
                print("is down, or we don't have an internet connection.")
            return None
        except Exception as e:
            print('An unexpected error occurred:', e)
            logging.error('An unexpected error occurred: %s' % e)
            return None
        else:
            return handle.read()


def usage():
    print("Usage: script.py -u <username> -p <password>")
    sys.exit(2)


def getHiddenRandHTMLResponse(response):
    # Implementazione del metodo getHiddenRandHTMLResponse
    target = '<input type="hidden" name="_token" value="'
    targetresponse = '<div id="sign-up-wrap"'

    # Assicurati che `response` sia una stringa
    if isinstance(response, bytes):
        response = response.decode('utf-8')

    start_index = response.find(targetresponse)
    if start_index == -1:
        return None

    parsedres = response[start_index:]
    start_index = parsedres.find(target)
    if start_index == -1:
        return None

    start_index += len(target)
    end_index = parsedres.find('"', start_index)

    if end_index == -1:
        return None

    return parsedres[start_index:end_index]


def checkLogin(response):
    # Implementazione del metodo checkLogin
    # Esempio: Verifica se la risposta contiene un testo specifico di successo
    return response  # "Login successful" in response


def main(argv):
    username = ""
    password = ""
    logfile = "/var/log/noipAutoLogin.log"
    hiddenval = ""
    theurl = "https://www.noip.com/login"
    thelogouturl = "https://www.noip.com/logout"
    txdata = None
    txheaders = {'User-agent': 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)'}
    # fake a user agent, some websites (like google) don't like automated exploration
    logging.basicConfig(filename=logfile, level=logging.DEBUG, format='%(asctime)s - %(levelname)s - %(message)s', datefmt='%Y/%m/%d %H:%M:%S')

    try:
        opts, args = getopt.getopt(argv, "hu:p:", ["help", "username=", "password="])
    except getopt.GetoptError:
        usage()
        logging.warning('Manual login with incorrect parameters')
        exit(2)
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            exit(2)
        elif opt in ("-u", "--username"):
            username = arg
        elif opt in ("-p", "--password"):
            password = arg

    myhtmlsession = HTMLSession(txheaders)
    response = myhtmlsession.openURI(theurl, None)
    if response is None:
        sys.exit(0)

    hiddenval = getHiddenRandHTMLResponse(response)
    txdata = urlencode({'username': username, 'password': password, 'Login': "1", 'submit_login_page': "1", '_token': hiddenval, 'Login': "Sign In"})
    response = myhtmlsession.openURI(theurl, txdata)
    if response is None:
        sys.exit(0)

    # we should sleep here for about 10 seconds.
    if checkLogin(response):
        print('We have succesfully logged into NoIP.')
        logging.info('We have succesfully logged into NoIP.')
    else:
        print('Login failed')
        logging.info('Login failed')

    response = myhtmlsession.openURI(thelogouturl, None)
    if response is None:
        sys.exit(0)




if __name__ == "__main__":
    main(sys.argv[1:])
